@@ -1,51 +1,35 @@
-//! Ptyprocess library provides an interface for a PTY/TTY.
+//! A library provides an interface for a unix [PTY/TTY](https://en.wikipedia.org/wiki/Pseudoterminal).
 //!
-//! The entry point to the library is a [PtyProcess].
-//!
-//! The library provides a `sync` and `async` IO operations for communication.
-//! To be able to use `async` you must to provide a feature flag `[async]`
-//! and turn off default features `default-features = false`.
+//! It aims to work on all major Unix variants.
 //!
 //! The library was developed as a backend for a https://github.com/zhiburt/expectrl.
 //! If you're interested in a high level operations may you'd better take a look at `zhiburt/expectrl`.
 //!
-//! # Example
+//! ## Usage
 //!
-//! ```no_run,ignore
+//! ```rust
 //! use ptyprocess::PtyProcess;
 //! use std::process::Command;
-//! use std::io::{Read, Write};
+//! use std::io::{BufRead, Write, BufReader};
 //!
 //! // spawn a cat process
 //! let mut process = PtyProcess::spawn(Command::new("cat")).expect("failed to spawn a process");
 //!
-//! // write message to cat.
-//! process.write_all(b"hello cat\n").expect("failed to write");
+//! // create a communication stream
+//! let mut stream = process.get_raw_handle().expect("failed to create a stream");
 //!
-//! // read what cat produced.
-//! let mut buf = vec![0; 128];
-//! let size = process.read(&mut buf).expect("failed to read");
-//! assert_eq!(&buf[..size], b"hello cat\r\n");
+//! // send a message to process
+//! writeln!(stream, "Hello cat").expect("failed to write to a stream");
 //!
-//! // stop process
-//! let sucess = process.exit(true).expect("failed to exit");
+//! // read a line from the stream
+//! let mut reader = BufReader::new(stream);
+//! let mut buf = String::new();
+//! reader.read_line(&mut buf).expect("failed to read a process output");
 //!
-//! assert_eq!(sucess, true);
-//! ```
+//! println!("line={}", buf);
 //!
-//! # Async
-//!
-//! ## Example
-//!
-//! ```no_run,ignore
-//! use ptyprocess::PtyProcess;
-//! use std::process::Command;
-//!
-//! // spawns a cat process
-//! let mut process = PtyProcess::spawn(Command::new("cat")).expect("failed to spawn a process");
-//!
-//! // sends line to cat
-//! process.send_line("hello cat").await.expect("failed writing");
+//! // stop the process
+//! assert!(process.exit(true).expect("failed to stop the process"))
 //! ```
 
 pub mod stream;
@@ -79,9 +63,12 @@ use termios::SpecialCharacterIndices;
 
 const DEFAULT_TERM_COLS: u16 = 80;
 const DEFAULT_TERM_ROWS: u16 = 24;
+
 const DEFAULT_VEOF_CHAR: u8 = 0x4; // ^D
 const DEFAULT_INTR_CHAR: u8 = 0x3; // ^C
 
+const DEFAULT_TERMINATE_DELAY: Duration = Duration::from_millis(100);
+
 /// PtyProcess controls a spawned process and communication with this.
 ///
 /// It implements [std::io::Read] and [std::io::Write] to communicate with
@@ -102,7 +89,7 @@ pub struct PtyProcess {
     child_pid: Pid,
     eof_char: u8,
     intr_char: u8,
-    terminate_approach_delay: Duration,
+    terminate_delay: Duration,
 }
 
 impl PtyProcess {
@@ -114,44 +101,42 @@ impl PtyProcess {
     ///     let proc = PtyProcess::spawn(Command::new("bash"));
     /// ```
     pub fn spawn(mut command: Command) -> Result<Self> {
-        let eof_char = get_eof_char();
-        let intr_char = get_intr_char();
-
         let master = Master::open()?;
         master.grant_slave_access()?;
         master.unlock_slave()?;
 
         // handle errors in child executions by pipe
-        let (exec_err_pipe_read, exec_err_pipe_write) = pipe()?;
+        let (exec_err_pipe_r, exec_err_pipe_w) = pipe()?;
 
         let fork = unsafe { fork()? };
         match fork {
             ForkResult::Child => {
                 let err = || -> Result<()> {
-                    let device = master.get_slave_name()?;
-                    let slave_fd = master.get_slave_fd()?;
-                    drop(master);
+                    make_controlling_tty(&master)?;
 
-                    make_controlling_tty(&device)?;
+                    let slave_fd = master.get_slave_fd()?;
                     redirect_std_streams(slave_fd)?;
 
                     set_echo(STDIN_FILENO, false)?;
                     set_term_size(STDIN_FILENO, DEFAULT_TERM_COLS, DEFAULT_TERM_ROWS)?;
 
-                    close(exec_err_pipe_read)?;
-                    // close pipe on sucessfull exec
-                    fcntl(exec_err_pipe_write, FcntlArg::F_SETFD(FdFlag::FD_CLOEXEC))?;
-
                     // Do not allow child to inherit open file descriptors from parent
-                    //
-                    // on linux could be used getrlimit(RLIMIT_NOFILE, rlim) interface
-                    let max_open_fds = sysconf(SysconfVar::OPEN_MAX)?.unwrap() as i32;
-                    // Why closing FD 1 causes an endless loop
-                    (3..max_open_fds)
-                        .filter(|&fd| fd != slave_fd && fd != exec_err_pipe_write)
-                        .for_each(|fd| {
-                            let _ = close(fd);
-                        });
+                    close_all_descriptors(&[
+                        0,
+                        1,
+                        2,
+                        slave_fd,
+                        exec_err_pipe_w,
+                        exec_err_pipe_r,
+                        master.as_raw_fd(),
+                    ])?;
+
+                    close(slave_fd)?;
+                    close(exec_err_pipe_r)?;
+                    drop(master);
+
+                    // close pipe on sucessfull exec
+                    fcntl(exec_err_pipe_w, FcntlArg::F_SETFD(FdFlag::FD_CLOEXEC))?;
 
                     let _ = command.exec();
                     Err(Error::last())
@@ -160,15 +145,18 @@ impl PtyProcess {
 
                 let code = err.as_errno().map_or(-1, |e| e as i32);
 
-                write(exec_err_pipe_write, &code.to_be_bytes())?;
+                // Intentionally ignoring errors to exit the process properly
+                let _ = write(exec_err_pipe_w, &code.to_be_bytes());
+                let _ = close(exec_err_pipe_w);
 
                 process::exit(code);
             }
             ForkResult::Parent { child } => {
-                close(exec_err_pipe_write)?;
+                close(exec_err_pipe_w)?;
 
                 let mut pipe_buf = [0u8; 4];
-                unistd::read(exec_err_pipe_read, &mut pipe_buf)?;
+                unistd::read(exec_err_pipe_r, &mut pipe_buf)?;
+                close(exec_err_pipe_r)?;
                 let code = i32::from_be_bytes(pipe_buf);
                 if code != 0 {
                     return Err(Error::from_errno(errno::from_i32(code)));
@@ -178,12 +166,15 @@ impl PtyProcess {
                 // that we need to set a terminal size in a parent.
                 set_term_size(master.as_raw_fd(), DEFAULT_TERM_COLS, DEFAULT_TERM_ROWS)?;
 
+                let eof_char = get_eof_char();
+                let intr_char = get_intr_char();
+
                 Ok(Self {
                     master,
                     child_pid: child,
                     eof_char,
                     intr_char,
-                    terminate_approach_delay: Duration::from_millis(100),
+                    terminate_delay: DEFAULT_TERMINATE_DELAY,
                 })
             }
         }
@@ -194,7 +185,10 @@ impl PtyProcess {
         self.child_pid
     }
 
-    /// Returns a file representation of a PTY, which can be used to communicate with it.
+    /// Returns a file representation of a PTY, which can be used
+    /// to communicate with a spawned process.
+    ///
+    /// The file behaivor is platform dependent.
     ///
     /// # Safety
     ///
@@ -202,9 +196,8 @@ impl PtyProcess {
     /// because it affects all structures which use it.
     ///
     /// Be carefull using this method in async mode.
-    /// Because descriptor is set to a non-blocking mode which may be unexpected.
-    ///
-    /// In future ut can be private for async feature if it will be considered an issue.
+    /// Because descriptor is set to a non-blocking mode will affect all dublicated descriptors
+    /// which may be unexpected.
     ///
     /// # Example
     ///
@@ -213,22 +206,29 @@ impl PtyProcess {
     /// use std::{process::Command, io::{BufReader, LineWriter}};
     ///
     /// let mut process = PtyProcess::spawn(Command::new("cat")).unwrap();
-    /// let pty = process.get_pty_handle().unwrap();
+    /// let pty = process.get_raw_handle().unwrap();
     /// let mut writer = LineWriter::new(&pty);
     /// let mut reader = BufReader::new(&pty);
     /// ```
-    pub fn get_pty_handle(&self) -> Result<File> {
+    pub fn get_raw_handle(&self) -> Result<File> {
         self.master.get_file_handle()
     }
 
+    /// Returns a stream representation of a PTY.
+    /// Which can be used to communicate with a spawned process.
+    ///
+    /// It differs from [Self::get_raw_handle] because it is
+    /// platform independent.
     pub fn get_pty_stream(&self) -> Result<Stream> {
-        self.get_pty_handle().map(Stream::new)
+        self.get_raw_handle().map(Stream::new)
     }
 
+    /// Get a end of file character if set or a default.
     pub fn get_eof_char(&self) -> u8 {
         self.eof_char
     }
 
+    /// Get a interapt character if set or a default.
     pub fn get_intr_char(&self) -> u8 {
         self.intr_char
     }
@@ -245,20 +245,6 @@ impl PtyProcess {
         set_term_size(self.master.as_raw_fd(), cols, rows)
     }
 
-    /// Waits until a echo settings is setup.
-    pub fn wait_echo(&self, on: bool, timeout: Option<Duration>) -> Result<bool> {
-        let now = time::Instant::now();
-        while timeout.is_none() || now.elapsed() < timeout.unwrap() {
-            if on == self.get_echo()? {
-                return Ok(true);
-            }
-
-            thread::sleep(Duration::from_millis(100));
-        }
-
-        Ok(false)
-    }
-
     /// The function returns true if an echo setting is setup.
     pub fn get_echo(&self) -> Result<bool> {
         termios::tcgetattr(self.master.as_raw_fd())
@@ -266,8 +252,9 @@ impl PtyProcess {
     }
 
     /// Sets a echo setting for a terminal
-    pub fn set_echo(&mut self, on: bool) -> Result<()> {
-        set_echo(self.master.as_raw_fd(), on)
+    pub fn set_echo(&mut self, on: bool, timeout: Option<Duration>) -> Result<bool> {
+        set_echo(self.master.as_raw_fd(), on)?;
+        self.wait_echo(on, timeout)
     }
 
     /// Returns true if a underline `fd` connected with a TTY.
@@ -276,8 +263,8 @@ impl PtyProcess {
     }
 
     /// Set the pty process's terminate approach delay.
-    pub fn set_terminate_approach_delay(&mut self, terminate_approach_delay: Duration) {
-        self.terminate_approach_delay = terminate_approach_delay;
+    pub fn set_terminate_delay(&mut self, terminate_approach_delay: Duration) {
+        self.terminate_delay = terminate_approach_delay;
     }
 
     /// Status returns a status a of child process.
@@ -366,10 +353,23 @@ impl PtyProcess {
 
     fn try_to_terminate(&mut self, signal: signal::Signal) -> Result<bool> {
         self.kill(signal)?;
-        thread::sleep(self.terminate_approach_delay);
+        thread::sleep(self.terminate_delay);
 
         self.is_alive().map(|is_alive| !is_alive)
     }
+
+    fn wait_echo(&self, on: bool, timeout: Option<Duration>) -> Result<bool> {
+        let now = time::Instant::now();
+        while timeout.is_none() || now.elapsed() < timeout.unwrap() {
+            if on == self.get_echo()? {
+                return Ok(true);
+            }
+
+            thread::sleep(Duration::from_millis(100));
+        }
+
+        Ok(false)
+    }
 }
 
 impl Drop for PtyProcess {
@@ -433,9 +433,25 @@ impl Master {
         get_slave_name(&self.fd)
     }
 
+    #[cfg(not(target_os = "freebsd"))]
     fn get_slave_fd(&self) -> Result<RawFd> {
         let slave_name = self.get_slave_name()?;
-        let slave_fd = open(slave_name.as_str(), OFlag::O_RDWR, Mode::empty())?;
+        let slave_fd = open(
+            slave_name.as_str(),
+            OFlag::O_RDWR | OFlag::O_NOCTTY,
+            Mode::empty(),
+        )?;
+        Ok(slave_fd)
+    }
+
+    #[cfg(target_os = "freebsd")]
+    fn get_slave_fd(&self) -> Result<RawFd> {
+        let slave_name = self.get_slave_name()?;
+        let slave_fd = open(
+            format!("/dev/{}", slave_name.as_str()).as_str(),
+            OFlag::O_RDWR | OFlag::O_NOCTTY,
+            Mode::empty(),
+        )?;
         Ok(slave_fd)
     }
 
@@ -453,11 +469,76 @@ impl AsRawFd for Master {
     }
 }
 
-#[cfg(not(target_os = "macos"))]
+#[cfg(target_os = "linux")]
 fn get_slave_name(fd: &PtyMaster) -> Result<String> {
     nix::pty::ptsname_r(fd)
 }
 
+#[cfg(target_os = "freebsd")]
+fn get_slave_name(fd: &PtyMaster) -> Result<String> {
+    use std::ffi::CStr;
+    use std::os::raw::c_char;
+    use std::os::unix::prelude::AsRawFd;
+
+    let fd = fd.as_raw_fd();
+
+    if !isptmaster(fd)? {
+        // never reached according current implementation of isptmaster
+        return Err(nix::Error::Sys(Errno::EINVAL));
+    }
+
+    // todo: Need to determine the correct size via some contstant like SPECNAMELEN in <sys/filio.h>
+    let mut buf: [c_char; 128] = [0; 128];
+
+    let _ = fdevname_r(fd, &mut buf)?;
+
+    // todo: determine how CStr::from_ptr handles not NUL terminated string.
+    let string = unsafe { CStr::from_ptr(buf.as_ptr()) }
+        .to_string_lossy()
+        .into_owned();
+
+    return Ok(string);
+}
+
+// https://github.com/freebsd/freebsd-src/blob/main/lib/libc/stdlib/ptsname.c#L52
+#[cfg(target_os = "freebsd")]
+fn isptmaster(fd: RawFd) -> Result<bool> {
+    use nix::libc::ioctl;
+    use nix::libc::TIOCPTMASTER;
+    match unsafe { ioctl(fd, TIOCPTMASTER as u64, 0) } {
+        0 => Ok(true),
+        _ => Err(Error::last()),
+    }
+}
+
+/* automatically generated by rust-bindgen 0.59.1 */
+// bindgen filio.h --allowlist-type fiodgname_arg -o bindings.rs
+// it may be worth to use a build.rs if we will need more FFI structures.
+#[cfg(target_os = "freebsd")]
+#[repr(C)]
+#[derive(Debug, Copy, Clone)]
+pub struct fiodgname_arg {
+    pub len: ::std::os::raw::c_int,
+    pub buf: *mut ::std::os::raw::c_void,
+}
+
+// https://github.com/freebsd/freebsd-src/blob/6ae38ab45396edaea26b4725e0c7db8cffa5f208/lib/libc/gen/fdevname.c#L39
+#[cfg(target_os = "freebsd")]
+fn fdevname_r(fd: RawFd, buf: &mut [std::os::raw::c_char]) -> Result<()> {
+    use nix::libc::{ioctl, FIODGNAME};
+
+    nix::ioctl_read_bad!(_ioctl_fiodgname, FIODGNAME, fiodgname_arg);
+
+    let mut fgn = fiodgname_arg {
+        len: buf.len() as i32,
+        buf: buf.as_mut_ptr() as *mut ::std::os::raw::c_void,
+    };
+
+    let _ = unsafe { _ioctl_fiodgname(fd, &mut fgn) }?;
+
+    Ok(())
+}
+
 /// Getting a slave name on darvin platform
 /// https://blog.tarq.io/ptsname-on-osx-with-rust/
 #[cfg(target_os = "macos")]
@@ -573,44 +654,83 @@ fn get_term_char(fd: RawFd, char: SpecialCharacterIndices) -> Result<u8> {
     Ok(b)
 }
 
-fn make_controlling_tty(child_name: &str) -> Result<()> {
-    // Is this appoach's result the same as just call ioctl TIOCSCTTY?
+fn make_controlling_tty(ptm: &Master) -> Result<()> {
+    #[cfg(not(any(target_os = "freebsd", target_os = "macos")))]
+    {
+        let pts_name = ptm.get_slave_name()?;
+        // https://github.com/pexpect/ptyprocess/blob/c69450d50fbd7e8270785a0552484182f486092f/ptyprocess/_fork_pty.py
 
-    // Disconnect from controlling tty, if any
-    let fd = open("/dev/tty", OFlag::O_RDWR | OFlag::O_NOCTTY, Mode::empty());
-    match fd {
-        Ok(fd) => {
-            close(fd)?;
+        // Disconnect from controlling tty, if any
+        //
+        // it may be a simmilar call to ioctl TIOCNOTTY
+        // https://man7.org/linux/man-pages/man4/tty_ioctl.4.html
+        let fd = open("/dev/tty", OFlag::O_RDWR | OFlag::O_NOCTTY, Mode::empty());
+        match fd {
+            Ok(fd) => {
+                close(fd)?;
+            }
+            Err(Error::Sys(Errno::ENXIO)) => {
+                // Sometimes we get ENXIO right here which 'probably' means
+                // that we has been already disconnected from controlling tty.
+                // Specifically it was discovered on ubuntu-latest Github CI platform.
+            }
+            Err(err) => return Err(err),
         }
-        Err(Error::Sys(Errno::ENXIO)) => {
-            // Sometimes we get ENXIO right here which 'probably' means
-            // that we has been already disconnected from controlling tty.
-            // Specifically it was discovered on ubuntu-latest Github CI platform.
+
+        // setsid() will remove the controlling tty. Also the ioctl TIOCNOTTY does this.
+        // https://www.win.tue.nl/~aeb/linux/lk/lk-10.html
+        setsid()?;
+
+        // Verify we are disconnected from controlling tty by attempting to open
+        // it again.  We expect that OSError of ENXIO should always be raised.
+        let fd = open("/dev/tty", OFlag::O_RDWR | OFlag::O_NOCTTY, Mode::empty());
+        match fd {
+            Err(Error::Sys(Errno::ENXIO)) => {} // ok
+            Ok(fd) => {
+                close(fd)?;
+                return Err(Error::UnsupportedOperation);
+            }
+            Err(_) => return Err(Error::UnsupportedOperation),
         }
-        Err(err) => return Err(err),
+
+        // Verify we can open child pty.
+        let fd = open(pts_name.as_str(), OFlag::O_RDWR, Mode::empty())?;
+        close(fd)?;
+
+        // Verify we now have a controlling tty.
+        let fd = open("/dev/tty", OFlag::O_WRONLY, Mode::empty())?;
+        close(fd)?;
     }
 
-    setsid()?;
+    #[cfg(any(target_os = "freebsd", target_os = "macos"))]
+    {
+        let pts_fd = ptm.get_slave_fd()?;
 
-    // Verify we are disconnected from controlling tty by attempting to open
-    // it again.  We expect that OSError of ENXIO should always be raised.
-    let fd = open("/dev/tty", OFlag::O_RDWR | OFlag::O_NOCTTY, Mode::empty());
-    match fd {
-        Err(Error::Sys(Errno::ENXIO)) => {} // ok
-        Ok(fd) => {
-            close(fd)?;
-            return Err(Error::UnsupportedOperation);
+        // https://docs.freebsd.org/44doc/smm/01.setup/paper-3.html
+        setsid()?;
+
+        use nix::libc::ioctl;
+        use nix::libc::TIOCSCTTY;
+        match unsafe { ioctl(pts_fd, TIOCSCTTY as u64, 0) } {
+            0 => {}
+            _ => return Err(Error::last()),
         }
-        Err(_) => return Err(Error::UnsupportedOperation),
     }
 
-    // Verify we can open child pty.
-    let fd = open(child_name, OFlag::O_RDWR, Mode::empty())?;
-    close(fd)?;
+    Ok(())
+}
 
-    // Verify we now have a controlling tty.
-    let fd = open("/dev/tty", OFlag::O_WRONLY, Mode::empty())?;
-    close(fd)?;
+// Except is used for cases like double free memory
+fn close_all_descriptors(except: &[RawFd]) -> Result<()> {
+    // On linux could be used getrlimit(RLIMIT_NOFILE, rlim) interface
+    let max_open_fds = sysconf(SysconfVar::OPEN_MAX)?.unwrap() as i32;
+    (0..max_open_fds)
+        .filter(|fd| !except.contains(fd))
+        .for_each(|fd| {
+            // We don't handle errors intentionally,
+            // because it will be hard to determine which descriptors closed already.
+            let _ = close(fd);
+        });
 
     Ok(())
 }
@@ -625,8 +745,21 @@ mod tests {
         master.grant_slave_access()?;
         master.unlock_slave()?;
         let slavename = master.get_slave_name()?;
-        assert!(slavename.starts_with("/dev"));
-        println!("slave name {}", slavename);
+
+        let expected_path = if cfg!(target_os = "freebsd") {
+            "pts/"
+        } else if cfg!(target_os = "macos") {
+            "/dev/ttys"
+        } else {
+            "/dev/pts/"
+        };
+
+        assert!(
+            slavename.starts_with(expected_path),
+            "pty_path=={}",
+            slavename
+        );
+
         Ok(())
     }
 
